@@ -48,7 +48,11 @@
 #endif
 
 
+#if RTLNX_VER_MIN(5,13,0)
+static inline struct vbox_private *vbox_bdev(struct ttm_device *bd)
+#else
 static inline struct vbox_private *vbox_bdev(struct ttm_bo_device *bd)
+#endif
 {
 	return container_of(bd, struct vbox_private, ttm.bdev);
 }
@@ -205,8 +209,13 @@ static int vbox_ttm_io_mem_reserve(struct ttm_bo_device *bdev,
 	return 0;
 }
 #else
+# if RTLNX_VER_MAX(5,13,0)
 static int vbox_ttm_io_mem_reserve(struct ttm_bo_device *bdev,
 				   struct ttm_resource *mem)
+# else /* > 5.13.0 */
+static int vbox_ttm_io_mem_reserve(struct ttm_device *bdev,
+				   struct ttm_resource *mem)
+# endif /* > 5.13.0 */
 {
 	struct vbox_private *vbox = vbox_bdev(bdev);
 	mem->bus.addr = NULL;
@@ -241,7 +250,12 @@ static int vbox_ttm_io_mem_reserve(struct ttm_bo_device *bdev,
 
 
 
-#if RTLNX_VER_MIN(5,10,0)
+#if RTLNX_VER_MIN(5,13,0)
+static void vbox_ttm_io_mem_free(struct ttm_device *bdev,
+				 struct ttm_resource *mem)
+{
+}
+#elif RTLNX_VER_MIN(5,10,0)
 static void vbox_ttm_io_mem_free(struct ttm_bo_device *bdev,
 				 struct ttm_resource *mem)
 {
@@ -253,7 +267,13 @@ static void vbox_ttm_io_mem_free(struct ttm_bo_device *bdev,
 }
 #endif
 
-#if RTLNX_VER_MIN(5,10,0)
+#if RTLNX_VER_MIN(5,13,0)
+static void vbox_ttm_tt_destroy(struct ttm_device *bdev, struct ttm_tt *tt)
+{
+	ttm_tt_fini(tt);
+	kfree(tt);
+}
+#elif RTLNX_VER_MIN(5,10,0)
 static void vbox_ttm_tt_destroy(struct ttm_bo_device *bdev, struct ttm_tt *tt)
 {
 	ttm_tt_fini(tt);
@@ -333,7 +353,11 @@ static int vbox_bo_move(struct ttm_buffer_object *bo, bool evict,
 }
 #endif
 
+#if RTLNX_VER_MIN(5,13,0)
+static struct ttm_device_funcs vbox_bo_driver = {
+#else /* < 5.13.0 */
 static struct ttm_bo_driver vbox_bo_driver = {
+#endif /* < 5.13.0 */
 	.ttm_tt_create = vbox_ttm_tt_create,
 #if RTLNX_VER_MIN(5,10,0)
 	.ttm_tt_destroy = vbox_ttm_tt_destroy,
@@ -370,14 +394,22 @@ int vbox_mm_init(struct vbox_private *vbox)
 {
 	int ret;
 	struct drm_device *dev = vbox->dev;
+#if RTLNX_VER_MIN(5,13,0)
+	struct ttm_device *bdev = &vbox->ttm.bdev;
+#else
 	struct ttm_bo_device *bdev = &vbox->ttm.bdev;
+#endif
 
 #if RTLNX_VER_MAX(5,0,0) && !RTLNX_RHEL_MAJ_PREREQ(7,7) && !RTLNX_RHEL_MAJ_PREREQ(8,1)
 	ret = vbox_ttm_global_init(vbox);
 	if (ret)
 		return ret;
 #endif
+#if RTLNX_VER_MIN(5,13,0)
+	ret = ttm_device_init(&vbox->ttm.bdev,
+#else
 	ret = ttm_bo_device_init(&vbox->ttm.bdev,
+#endif
 #if RTLNX_VER_MAX(5,0,0) && !RTLNX_RHEL_MAJ_PREREQ(7,7) && !RTLNX_RHEL_MAJ_PREREQ(8,1)
 				 vbox->ttm.bo_global_ref.ref.object,
 #endif
@@ -388,7 +420,7 @@ int vbox_mm_init(struct vbox_private *vbox)
 #if RTLNX_VER_MIN(3,15,0) || RTLNX_RHEL_MAJ_PREREQ(7,1)
 				 dev->anon_inode->i_mapping,
 #endif
-#if RTLNX_VER_MIN(5,5,0) || RTLNX_RHEL_MIN(8,3)
+#if RTLNX_VER_MIN(5,5,0) || RTLNX_RHEL_MIN(8,3) || RTLNX_SUSE_MAJ_PREREQ(15,3)
 				 dev->vma_offset_manager,
 #elif RTLNX_VER_MAX(5,2,0) && !RTLNX_RHEL_MAJ_PREREQ(8,2)
 				 DRM_FILE_PAGE_OFFSET,
@@ -429,7 +461,11 @@ int vbox_mm_init(struct vbox_private *vbox)
 	return 0;
 
 err_device_release:
+#if RTLNX_VER_MIN(5,13,0)
+	ttm_device_fini(&vbox->ttm.bdev);
+#else
 	ttm_bo_device_release(&vbox->ttm.bdev);
+#endif
 #if RTLNX_VER_MAX(5,0,0) && !RTLNX_RHEL_MAJ_PREREQ(7,7) && !RTLNX_RHEL_MAJ_PREREQ(8,1)
 err_ttm_global_release:
 	vbox_ttm_global_release(vbox);
@@ -446,7 +482,11 @@ void vbox_mm_fini(struct vbox_private *vbox)
 #else
 	arch_phys_wc_del(vbox->fb_mtrr);
 #endif
+#if RTLNX_VER_MIN(5,13,0)
+	ttm_device_fini(&vbox->ttm.bdev);
+#else
 	ttm_bo_device_release(&vbox->ttm.bdev);
+#endif
 #if RTLNX_VER_MAX(5,0,0) && !RTLNX_RHEL_MAJ_PREREQ(7,7) && !RTLNX_RHEL_MAJ_PREREQ(8,1)
 	vbox_ttm_global_release(vbox);
 #endif
@@ -528,7 +568,9 @@ int vbox_bo_create(struct drm_device *dev, int size, int align,
 {
 	struct vbox_private *vbox = dev->dev_private;
 	struct vbox_bo *vboxbo;
+#if RTLNX_VER_MAX(5,13,0)
 	size_t acc_size;
+#endif
 	int ret;
 
 	vboxbo = kzalloc(sizeof(*vboxbo), GFP_KERNEL);
@@ -551,16 +593,20 @@ int vbox_bo_create(struct drm_device *dev, int size, int align,
 
 	vbox_ttm_placement(vboxbo, VBOX_MEM_TYPE_VRAM | VBOX_MEM_TYPE_SYSTEM);
 
+#if RTLNX_VER_MAX(5,13,0)
 	acc_size = ttm_bo_dma_acc_size(&vbox->ttm.bdev, size,
 				       sizeof(struct vbox_bo));
+#endif
 
 	ret = ttm_bo_init(&vbox->ttm.bdev, &vboxbo->bo, size,
 			  ttm_bo_type_device, &vboxbo->placement,
 #if RTLNX_VER_MAX(4,17,0) && !RTLNX_RHEL_MAJ_PREREQ(7,6) && !RTLNX_SUSE_MAJ_PREREQ(15,1) && !RTLNX_SUSE_MAJ_PREREQ(12,5)
 			  align >> PAGE_SHIFT, false, NULL, acc_size,
-#else
+#elif RTLNX_VER_MAX(5,13,0) /* < 5.13.0 */
 			  align >> PAGE_SHIFT, false, acc_size,
-#endif
+#else /* > 5.13.0 */
+			  align >> PAGE_SHIFT, false,
+#endif /* > 5.13.0 */
 #if RTLNX_VER_MIN(3,18,0) || RTLNX_RHEL_MAJ_PREREQ(7,2)
 			  NULL, NULL, vbox_bo_ttm_destroy);
 #else
@@ -580,7 +626,7 @@ err_free_vboxbo:
 
 static inline u64 vbox_bo_gpu_offset(struct vbox_bo *bo)
 {
-#if RTLNX_VER_MIN(5,9,0) || RTLNX_RHEL_MIN(8,4)
+#if RTLNX_VER_MIN(5,9,0) || RTLNX_RHEL_MIN(8,4) || RTLNX_SUSE_MAJ_PREREQ(15,3)
 	return bo->bo.mem.start << PAGE_SHIFT;
 #else
 	return bo->bo.offset;
